anim8 = require('libs/anim8/anim8')

Player = Class{}

-- sprites tiles sizes
local RUN_TILE_WIDTH, RUN_TILE_HEIGHT = 104, 128
local JUMP_TILE_WIDTH, JUMP_TILE_HEIGHT = 97, 128
local SLIDE_TILE_WIDTH, SLIDE_TILE_HEIGHT = 84, 83
local DEAD_TILE_WIDTH, DEAD_TILE_HEIGHT = 125, 128

-- sprite sheets and their grid animation
local spriteSheets = {}

spriteSheets.runImg = g.newImage('assets/graphics/player/runSpriteSheetScaled.png')
spriteSheets.runGrid = anim8.newGrid(RUN_TILE_WIDTH, RUN_TILE_HEIGHT, spriteSheets.runImg:getWidth(), spriteSheets.runImg:getHeight())
spriteSheets.jumpImg = g.newImage('assets/graphics/player/jumpSpriteSheetScaled.png')
spriteSheets.jumpGrid = anim8.newGrid(JUMP_TILE_WIDTH, JUMP_TILE_HEIGHT, spriteSheets.jumpImg:getWidth(), spriteSheets.jumpImg:getHeight())
spriteSheets.slideImg = g.newImage('assets/graphics/player/slideSpriteSheetScaled.png')
spriteSheets.slideGrid = anim8.newGrid(SLIDE_TILE_WIDTH, SLIDE_TILE_HEIGHT, spriteSheets.slideImg:getWidth(), spriteSheets.slideImg:getHeight())
spriteSheets.deadImg = g.newImage('assets/graphics/player/deadSpriteSheetScaled.png')
spriteSheets.deadGrid = anim8.newGrid(DEAD_TILE_WIDTH, DEAD_TILE_HEIGHT, spriteSheets.deadImg:getWidth(), spriteSheets.deadImg:getHeight())

-- sounds
local sounds = {
  jump = love.audio.newSource('assets/audio/jump2.wav', 'static'),
  collect = love.audio.newSource('assets/audio/sfx_select.wav', 'static'),
  slide = love.audio.newSource('assets/audio/slide.wav', 'static'),
  die = love.audio.newSource('assets/audio/death.wav', 'static')
}


function Player:init()

  -- animations
  ANIMATION_SPEED = 0.04
  self.runAni = anim8.newAnimation(spriteSheets.runGrid('1-10', 1), ANIMATION_SPEED)
  self.jumpAni = anim8.newAnimation(spriteSheets.jumpGrid('1-10', 1), ANIMATION_SPEED)
  self.slideAni = anim8.newAnimation(spriteSheets.slideGrid('1-10', 1), ANIMATION_SPEED)
  self.deadAni = anim8.newAnimation(spriteSheets.deadGrid('1-10', 1), ANIMATION_SPEED)

  -- positioning
  self.x = 30
  self.y = WINDOW_HEIGHT - ground.h - RUN_TILE_HEIGHT

  -- physics
  self.ground = self.y  -- if y changes, we can use this to go to ground
  self.dy = 0
  self.maxJump = -650
  self.gravity = -1800

  -- player states, initialize as run
  self.state = 'run'
  self.prevState = self.state -- so we can change player position when deadd

  -- timer for the slide and dying animation so we can stop them
  slideTimer = 0
  dyingTimer = 0

  -- to update the current animation and the image of the animation when player state changes
  -- initialize to run
  currentAni = {}
  currentAni.img = spriteSheets.runImg
  currentAni.ani = self.runAni

  currentTileDimensions = {RUN_TILE_WIDTH, RUN_TILE_HEIGHT} -- so we can detect collision

  self.score = 0

end


function Player:update(dt)

  -- if player dead, play animation until we reached the timer then
  if self.state == 'dead' then
    sounds.die:play()
    dyingTimer = dyingTimer + dt
    if dyingTimer <= .4 then
      currentAni.ani:update(dt)

    -- after dead animation finishes, go to finish state
    elseif dyingTimer >= 1 then
      gameState = 'finish'
    end
  else

  -- update animation depending on the state of the plyer
  currentAni.ani:update(dt)

  -- jump if space is pressed
  if keyWasPressed('space') and self.dy == 0 then
    self:jump(dt)
  end

  -- if we hit s, then slide, but make sure he is not in the air
  if keyWasPressed('s') and self.state ~= 'jump' then
    self:slide(dt)
  end

  -- apply gravity if we are not in the ground
  if self.dy ~= 0 then
    self.y = self.y + self.dy *dt
    self.dy = self.dy - self.gravity * dt
  end

  -- reset dy and y to the ground and make sure he's not sliding
  if self.y > self.ground and self.state ~= 'slide' then
    self.state = 'run'
    self.prevState = self.state
    currentTileDimensions = {RUN_TILE_WIDTH, RUN_TILE_HEIGHT}
    currentAni.img = spriteSheets.runImg
    currentAni.ani = self.runAni
    self.dy = 0
    self.y = self.ground
  end

  -- limit sliding time then reset to the running state
  if self.state == 'slide' then
  slideTimer = slideTimer + dt

    if slideTimer >= .6 then
      self:run(dt)
    end
  end

  -- loop obstacles and check collision
  for i = #map.obstacles, 1, -1 do

    local o = map.obstacles[i]

    -- handel the fork trap and the brick trap being lower than the tile height
    local objY = o.y
    local playerY = self.y
    if o.id == FORK_TRAP or o.id == BRICK_TRAP then
      objY = o.y + (MAP_TILE_HEIGHT / 2)
    end

    -- handle player's sliding being close lower
    if self.state == 'slide' then
      playerY = self.y + currentTileDimensions[1] / 2
    end

    -- check collision
    if CheckCollision(self.x, playerY, currentTileDimensions[1], currentTileDimensions[2],
                      o.x, objY, MAP_TILE_WIDTH, MAP_TILE_HEIGHT) then

      -- if gold add 10 to score
      if o.id == GOLD_TROPHY then
        sounds.collect:play()
        table.remove(map.obstacles, i)
        self.score = self.score + 10

      --other trophies 2
      elseif o.id == BLUE_TROPHY or
             o.id == GREEN_TROPHY or
             o.id == RED_TROPHY then

        sounds.collect:play()
        table.remove(map.obstacles, i)
        self.score = self.score + 2

      -- if hit obstacle, game over
      else
        self.state = 'dead'
        -- stop screen scrolling
        bg.scrollingSpeed = 0
        ground.scrollingSpeed = 0
        currentAni.img = spriteSheets.deadImg
        currentAni.ani = self.deadAni

        -- when laying down, we need to shift his y position down to match the ground
        if self.prevState == 'jump' then
          self.y = self.ground + (currentTileDimensions[2] / 6) -- this number is chosen by trial and error
        elseif self.prevState == 'run' then
          self.y = self.y + (currentTileDimensions[2] / 6 )
        elseif self.prevState == 'slide' then
          self.y = self.y - (currentTileDimensions[2] / 3)
        end
      end
    end

  end
end



end


function Player:render()
  currentAni.ani:draw(currentAni.img, self.x, self.y)
end

-- changes the player to the jump state and apply the jump physics
function Player:jump(dt)
  self.state = 'jump'
  sounds.jump:play()
  self.prevState = self.state
  currentTileDimensions = {JUMP_TILE_WIDTH, JUMP_TILE_HEIGHT}
  currentAni.img = spriteSheets.jumpImg
  currentAni.ani = self.jumpAni
  self.dy = self.maxJump
end


-- changes the player to the slide state and apply slide changes
function Player:slide(dt)
  self.state = 'slide'
  sounds.slide:play()
  self.prevState = self.state
  currentTileDimensions = {SLIDE_TILE_WIDTH, SLIDE_TILE_HEIGHT}
  currentAni.img = spriteSheets.slideImg
  currentAni.ani = self.slideAni

  -- also we need to lower the image so
  -- he goes into the ground and doesn't slide in the air
  self.y = self.y + RUN_TILE_HEIGHT/3
end

-- changes the player to the running state
function Player:run(dt)
  self.state = 'run'
  self.prevState = self.state
  currentTileDimensions = {RUN_TILE_WIDTH, RUN_TILE_HEIGHT}
  currentAni.img = spriteSheets.runImg
  currentAni.ani = self.runAni
  slideTimer = 0
end